// Volatility Workbecnch software
// Spawn with redirect utility code
// Copyright Wrensoft 2017
// www.passmark.com
//
// Based on Microsoft Knowledge Base Article 190351,
// "HOWTO: Spawn Console Processes with Redirected Standard Handles"

#include<windows.h>
#include <stdio.h>
#include "VolatilityWorkbench.h"

void DisplayError(char *pszAPI);
void PrepAndLaunchRedirectedChild(HANDLE hChildStdOut, HANDLE hChildStdErr, wchar_t *ProgName, PROCESS_INFORMATION *ProcessInfo);

//********************************
////SpawnWithRedirect
//
//Spawn a process with the stdout of the child
//process redirected
//********************************
HANDLE SpawnWithRedirect(wchar_t *ProgName, PROCESS_INFORMATION *ProcessInfo)
{
	HANDLE hOutputReadTmp,hOutputRead,hOutputWrite;
	HANDLE hErrorWrite;
	SECURITY_ATTRIBUTES sa;
	
	
	// Set up the security attributes struct.
	sa.nLength= sizeof(SECURITY_ATTRIBUTES);
	sa.lpSecurityDescriptor = NULL;
	sa.bInheritHandle = TRUE;
	
	
	// Create the child output pipe.
	if (!CreatePipe(&hOutputReadTmp,&hOutputWrite,&sa,0))
		DisplayError("CreatePipe");
	
	
	// Create a duplicate of the output write handle for the std error
	// write handle. This is necessary in case the child application
	// closes one of its std output handles.
	if (!DuplicateHandle(GetCurrentProcess(),hOutputWrite,
		GetCurrentProcess(),&hErrorWrite,0,
		TRUE,DUPLICATE_SAME_ACCESS))
		DisplayError("DuplicateHandle");
	
	
	// Create new output read handle and the input write handles. Set
	// the Properties to FALSE. Otherwise, the child inherits the
	// properties and, as a result, non-closeable handles to the pipes
	// are created.
	if (!DuplicateHandle(GetCurrentProcess(),hOutputReadTmp,
		GetCurrentProcess(),
		&hOutputRead, // Address of new handle.
		0,FALSE, // Make it uninheritable.
		DUPLICATE_SAME_ACCESS))
		DisplayError("DupliateHandle");
	

	// Close inheritable copies of the handles you do not want to be
	// inherited.
	if (!CloseHandle(hOutputReadTmp)) DisplayError("CloseHandle");
	
	PrepAndLaunchRedirectedChild(hOutputWrite, hErrorWrite, ProgName, ProcessInfo);
		
	// Close pipe handles (do not continue to modify the parent).
	// You need to make sure that no handles to the write end of the
	// output pipe are maintained in this process or else the pipe will
	// not close when the child process exits and the ReadFile will hang.
	if (!CloseHandle(hOutputWrite)) DisplayError("CloseHandle");
	if (!CloseHandle(hErrorWrite)) DisplayError("CloseHandle");
	
	// Redirection is complete
	return (hOutputRead);
	
}


/////////////////////////////////////////////////////////////////////// 
// PrepAndLaunchRedirectedChild
// Sets up STARTUPINFO structure, and launches redirected child.
/////////////////////////////////////////////////////////////////////// 
void PrepAndLaunchRedirectedChild(HANDLE hChildStdOut,
								  HANDLE hChildStdErr,
								  wchar_t *ProgName, PROCESS_INFORMATION *ProcessInfo)
{
	STARTUPINFO			si;
	char				ErrorStr[512];
	
	// Set up the start up info struct.
	ZeroMemory(&si,sizeof(STARTUPINFO));
	si.cb = sizeof(STARTUPINFO);
	// Hide the child window (must include STARTF_USESHOWWINDOW)
	si.wShowWindow = SW_HIDE;
	si.dwFlags = STARTF_USESTDHANDLES | STARTF_USESHOWWINDOW;
	si.hStdOutput = hChildStdOut;
	si.hStdInput  = NULL;
	si.hStdError  = hChildStdErr;
	
	// Launch the process that you want to redirect (in this case,
	// Child.exe). Make sure Child.exe is in the same directory as
	// redirect.c launch redirect from a command line to prevent location
	// confusion.
	if (!CreateProcess(NULL, ProgName, NULL, NULL, TRUE, CREATE_NEW_CONSOLE | CREATE_NO_WINDOW, NULL, NULL, &si, ProcessInfo)) {
		sprintf_s (ErrorStr, 512, "CreateProcess failed\n%s",ProgName);
		DisplayError(ErrorStr);
		
	}
	//else {
	//	// Close any unnecessary handles.
	//	if (!CloseHandle(ProcessInfo->hThread)) DisplayError("CloseHandle");
	//	if (!CloseHandle(ProcessInfo.hProcess)) DisplayError("CloseHandle");
	//}
}



/////////////////////////////////////////////////////////////////////// 
// DisplayError
// Displays the error number and corresponding message.
/////////////////////////////////////////////////////////////////////// 
void DisplayError(char *pszAPI)
{
	LPVOID lpvMessageBuffer;
	wchar_t szPrintBuffer[512];
	
	FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,
		NULL, GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		(LPTSTR)&lpvMessageBuffer, 0, NULL);
	
	swprintf_s (szPrintBuffer, 512,
		L"ERROR: API    = %s\nError code = %d.\nMessage   = %s\n",
		pszAPI, GetLastError(), (char *)lpvMessageBuffer);
	
	MessageBox (ghMainWnd, szPrintBuffer, L"Error while starting search", MB_ICONERROR | MB_OK);

	LocalFree(lpvMessageBuffer);
}

