﻿// Volatility Workbecnch software
// Main header file
// Copyright Wrensoft 2018
// www.passmark.com
//
// Volatility Workbecnch is free software; you can redistribute it and / or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Volatility Workbecnch is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Volatility.If not, see <http://www.gnu.org/licenses/>.

#ifndef VOLATILITY_WORKBENCH_H
#define VOLATILITY_WORKBENCH_H

#include "resource.h"

#define	COMPANY_NAME				L"PassMark"
#define	COMPANY_NAME_SHORT			L"PassMark"

#define PROGRAM_NAME				L"Volatility Workbench"
#define PROGRAM_VERSION				L"V2.1"
#define PROGRAM_BUILD				L"1000"
#define	PASSMARK_WEBSITE			L"http://www.passmark.com"
#define VOLATILITY_EXE				L"volatility.exe"
#define VOLATILITY_VER				L"2.6.1"
#define	VOLATILITY_WEBSITE			L"http://www.volatilityfoundation.org"

#define MIN_WINDOW_WIDTH			1090	//Minimum dialog width
#define MIN_WINDOW_HEIGHT			700		//Minimum dialog height
#define WEBCTRL_OFFSET_X			15		//X Offset of web control in the dialog
#define WEBCTRL_OFFSET_Y			210		//Y Offset of web cotnrol int the dialog
#define WEBCTRL_WIDTHOFFSET_X		30		//Offset for width of web control 
#define WEBCTRL_HEIGHTOFFSET_X		260		//Offset for height of web control 

#define MAXDIRNAME					300
#define	MAXFILENAME					200
#define	MAXPARAMETERSLEN			500
#define MAX_SEARCH_STRING_LEN		300		//maximum search string lenght
#define MAX_ENVIRONMENT_STRING_LEN	2024
#define MAX_URL_LEN					2048
#define MAX_HEADER_TEXT_LEN			1024
#define MAX_RESULT_BUFF_LEN			1048576
#define	INIT_HTML_BUFF_LEN			1024
#define OK							true
#define NOK							false
#define NUM_OF_FILES             150

//Window resizing modes
#define		RS_MOVE			0
#define		RS_RESIZE		1
#define		RS_MOVE_DOWN	2
#define		RS_MOVE_ACROSS	3
//#define		RS_RESIZE_HALFY	4
#define		RS_MOVE_ACROSS_SIZE_DOWN	5
#define		RS_MOVE_DOWN_SIZE_ACROSS	6
#define		RS_MOVE_DOWN_AND_ACROSS	7
#define		RS_SIZE_ACROSS	8

//Image positioning
#define		IMAGE_OFFSET	0
#define		IMAGE_HEIGHT	102

#define     TOT_COMMANDS           161
#define		NUM_WIN_COMMANDS	   79
#define		NUM_MAC_COMMANDS	   38
#define		NUM_LIN_COMMANDS	   44
#define		NUM_PROFILES		   200
#define     NUM_BASE_PROFILES       54
#define		MAX_NUM_PARAM		   10
#define		MAX_KDBG_ADDR_LEN	   20
#define		MAX_COM_STR_LEN	       10
#define		MAX_NUM_PROCESS		   500
#define		MAX_PROCESS_LEN		   50

#define		MAX_NUM_VALUEITEMS		500
#define		MAX_VALUE_LEN			50

#define		CHECKTYPE_NOCHECKBOX	0
#define		CHECKTYPE_CHECKBOX		1
#define		CHECKTYPE_RADIOBUTTON	2

#define		VALUETYPE_NOVALUE		0
#define		VALUETYPE_EDITBOX		1
#define		VALUETYPE_COMBOBOX		2

#define		VALIDATION_DISABLED		0
#define		VALIDATION_DIGIT		1
#define		VALIDATION_HEXADDRESS	2
#define		VALIDATION_REGEX		3
#define		VALIDATION_DIR			4

typedef enum CURSOR_TYPE
{
	ARROW = 0,
	WAIT
};

typedef struct ParamValue_t
{
	int			NumItems;
	wchar_t		Name[MAX_NUM_VALUEITEMS][MAX_VALUE_LEN];
	wchar_t		Value[MAX_NUM_VALUEITEMS][MAX_VALUE_LEN];
};

struct VOLATILITY_PARAM_TYPEDEF
{
	wchar_t			*pszLabelText;
	wchar_t			*pszParam;
	char			CheckType;
	int				nCheckDlgID;
	bool			bChecked;
	char			ValueType;
	int				nValueDlgID;
	ParamValue_t	*ValueList;
	char			ValidationType;
};

#define CMD_PARAM (_LABELTXT, _PARAM, _CHECKBOXTYPE, _CHECKID, _CHECKED, _VALUETYPE, _VALUEID) {_LABELTXT, _PARAM, _CHECKBOXTYPE, _CHECKID, _CHECKED, _VALUETYPE, _VALUEID}
#define NOPARAM		{L"", L"", CHECKTYPE_NOCHECKBOX, 0, 0, VALUETYPE_NOVALUE, 0}

typedef struct VOLATILITY_CMD_TYPEDEF
{
	wchar_t		*pszCommand;
	wchar_t		*pszDescription;
	int			nParams;
	VOLATILITY_PARAM_TYPEDEF	params[MAX_NUM_PARAM];
};

#define VOLATILITY_COMMAND(_CMD, _DESCRIPTION, _NUMPARAM, _PARAM1, _PARAM2, _PARAM3, _PARAM4, _PARAM5, _PARAM6, _PARAM7, _PARAM8, _PARAM9, _PARAM10)	{ _CMD, _DESCRIPTION, _NUMPARAM, { _PARAM1, _PARAM2, _PARAM3, _PARAM4, _PARAM5, _PARAM6, _PARAM7, _PARAM8, _PARAM9, _PARAM10 } }

typedef struct VOLATILITY_PROFILE_TYPEDEF
{
	wchar_t *Profile;
	wchar_t *Description;
};

#define VOLATILITY_PROFILE(PROFILE, DESCRIPTION)	{PROFILE, DESCRIPTION}



typedef struct CONFIG
{
	wchar_t		Profile[MAX_PROFILE_LEN];
	wchar_t		KDBG_Addr[MAX_KDBG_ADDR_LEN];
	int			ProcessCount;
	wchar_t		ProcessName[MAX_NUM_PROCESS][MAX_PROCESS_LEN];
	wchar_t		PID[MAX_NUM_PROCESS][MAX_PROCESS_LEN];
};

// Global variables
#if defined (MAIN_MODULE)
	HWND				ghMainWnd;					//Main window
	char				lastURL[MAX_URL_LEN];		//last URL that was clicked and needs processing
	wchar_t				fullpath[MAXFILENAME + MAXDIRNAME] = { L'\0' };		//Work area, temp var
	wchar_t				RunDirName[MAXDIRNAME];
	wchar_t				szDefaultLogName[MAXFILENAME] = L"\\VolatilityWorkbenchLog.txt";	//Contains the default Log file name
	wchar_t				gFileName[MAXFILENAME + MAXDIRNAME];
	unsigned int		gControlsList[] = { IDC_CHECK1, IDC_CHECK2, IDC_CHECK3, IDC_CHECK4, IDC_CHECK5, IDC_CHECK6, IDC_CHECK7, IDC_CHECK8, IDC_CHECK9, IDC_CHECK10, IDC_RADIO1, IDC_RADIO2, IDC_RADIO3, IDC_RADIO4, IDC_RADIO5, IDC_RADIO6, IDC_EDIT1, IDC_EDIT2, IDC_EDIT3, IDC_EDIT4, IDC_EDIT5, IDC_EDIT6, IDC_DROPDOWN1, IDC_DROPDOWN2, IDC_DROPDOWN3, IDC_DROPDOWN4, IDC_DROPDOWN5, IDC_DROPDOWN6, IDC_GROUP1, IDC_GROUP2, IDC_GROUP3, IDC_GROUP4, IDC_GROUP5, IDC_GROUP6, IDC_GROUP7, IDC_GROUP8};
	ParamValue_t		gProcessList;

	VOLATILITY_CMD_TYPEDEF		gCommands[TOT_COMMANDS] =
	{
		{ L"-- Hunting rootkits and malicious code --", L"", 0 },

		{ L"malfind", L"Find hidden and injected code", 5,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX },
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_DIR },
			{ L"Maximum size", L"--max-size=", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT5, NULL, VALIDATION_DIGIT }
		}
		},

		{ L"yarascan", L"Scan process or kernel memory with Yara signatures", 10,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX },
			{ L"Yara rules (String)", L"--yara-rules=", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_CHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_DISABLED },
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT5, NULL, VALIDATION_DIR },
			{ L"Maximum size", L"--max-size=", CHECKTYPE_CHECKBOX, IDC_CHECK6, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT6, NULL, VALIDATION_DIGIT },
			{ L"Scan process and kernel mem", L"--all", CHECKTYPE_CHECKBOX, IDC_CHECK7, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Case insensitive", L"--case", CHECKTYPE_CHECKBOX, IDC_CHECK8, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Scan kernel modules", L"--kernel", CHECKTYPE_CHECKBOX, IDC_CHECK9, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Match wide", L"--wide", CHECKTYPE_CHECKBOX, IDC_CHECK10, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"svcscan", L"Scan for Windows services", 1,
		{

			{ L"Verbose", L"--verbose", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"ldrmodules", L"Detect unlinked DLLs", 4,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX },
			{ L"Verbose", L"--verbose", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"impscan", L"Scan for calls to imported functions", 4,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Base Address", L"--base=", CHECKTYPE_RADIOBUTTON, IDC_CHECK3, BST_CHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_HEXADDRESS },
			{ L"Scan size", L"--size=", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_DIGIT }
		}
		},

		{ L"apihooks", L"Detect API hooks in process and kernel memory", 9,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX },
			{ L"Bypass sanity checks", L"--unsafe", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"No whitelist", L"--no-whitelist", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Skip process checks", L"--skip-process", CHECKTYPE_CHECKBOX, IDC_CHECK6, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Skip kernel mode checks", L"--skip-kernel", CHECKTYPE_CHECKBOX, IDC_CHECK7, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Modify base to in-mem base address", L"--fix", CHECKTYPE_CHECKBOX, IDC_CHECK9, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Fast mode", L"--quick", CHECKTYPE_CHECKBOX, IDC_CHECK10, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
		}
		},

		{ L"idt", L"Display Interrupt Descriptor Table", 1,
		{

			{ L"Verbose", L"--verbose", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"gdt", L"Display Global Descriptor Table", 0 },

		{ L"threads", L"Investigate _ETHREAD and _KTHREADs", 5,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX },
			{ L"Tags to filter", L"--filter=", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_DISABLED },
			{ L"List all tags", L"--listtags", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"callbacks", L"Print system-wide notification routines", 4,
		{
			{ L"Start scaning address", L"--start=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
			{ L"Length (in bytes)", L"--length=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DIGIT },
			{ L"Scan virtual space", L"--virtual", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Skip unalloc objects", L"--show-unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"driverirp", L"Driver IRP hook detection", 6,
		{
			{ L"Driver Name (Regex)", L"--regex=", CHECKTYPE_RADIOBUTTON, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_REGEX },
			{ L"Start scaning address", L"--start=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Length (in bytes)", L"--length=", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_DIGIT },
			{ L"Scan virtual space", L"--virtual", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Skip unalloc objects", L"--show-unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Verbose", L"--verbose", CHECKTYPE_CHECKBOX, IDC_CHECK6, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"devicetree", L"Show device tree", 4,
		{
			{ L"Start scaning address", L"--start=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
			{ L"Length (in bytes)", L"--length=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DIGIT },
			{ L"Scan virtual space", L"--virtual", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Skip unalloc objects", L"--show-unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"psxview", L"Find hidden processes with various process listings", 2,
		{
			{ L"Physical Offset", L"--physical-offset", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Apply known rules", L"--apply-rules", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"timers", L"Print kernel timers and associated module DPCs", 1,
		{
			{ L"nt!KiTimer... Address", L"--listhead=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
		}
		},

		{ L"-- Processes and DLLs --", L"", 0 },
		
		{ L"pslist", L"Print all running processes by following the EPROCESS\r\nlists", 4, 
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX },
			{ L"Display physical offsets", L"--physical-offset", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		} 
		},

		{ L"pstree", L"Print process list as a tree", 0 },

		{ L"psscan", L"Pool scanner for process objects", 4,
		{
			{ L"Start scaning address", L"--start=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
			{ L"Length (in bytes)", L"--length=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DIGIT },
			{ L"Scan virtual space", L"--virtual", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Skip unalloc objects", L"--show-unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"dlldump", L"Dump DLLs from a process address space\r\nDump directory will be created in the Volatility Workbench directory", 10,
		{
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_CHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_DIR },
			{ L"Dlls Name (Regex)", L"--regex=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_REGEX },
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN3, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO4, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO5, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT5, NULL, VALIDATION_REGEX },
			{ L"Dump at base offset", L"--base=", CHECKTYPE_CHECKBOX, IDC_CHECK6, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT6, NULL, VALIDATION_HEXADDRESS },
			{ L"Bypass certain sanity checks", L"--unsafe", CHECKTYPE_CHECKBOX, IDC_CHECK7, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Ignore case", L"--ignore-case", CHECKTYPE_CHECKBOX, IDC_CHECK8, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Modify base to in-mem base address", L"--fix", CHECKTYPE_CHECKBOX, IDC_CHECK9, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Carve memory sample", L"--memory", CHECKTYPE_CHECKBOX, IDC_CHECK10, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"handles", L"Dump the USER handle tables", 6,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX },
			{ L"Objects (comma-sep)", L"--object-type=", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_DISABLED },
			{ L"Physical offset", L"--physical-offset", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Silent", L"--silent", CHECKTYPE_CHECKBOX, IDC_CHECK6, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"getsids", L"Print the SIDs owning each process", 3,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX }
		}
		},

		{ L"cmdscan", L"Extract command history by scanning for _COMMAND_HISTORY", 3,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX }
		}
		},

		{ L"consoles", L"Extract command history by scanning for _CONSOLE_INFORMATION", 3,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX }
		}
		},

		{ L"privs", L"Display process privileges", 5,
		{
			{ L"Privileges (Regex)", L"--regex=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_REGEX },
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN2, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO4, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_REGEX },
			{ L"Silent", L"--silent", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }

		}
		},

		{ L"envars", L"Display process environment variables", 4,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX },
			{ L"Silent", L"--silent", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }

		}
		},

		{ L"verinfo", L"Prints out the version information from PE images", 7,
		{
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_DIR },
			{ L"Dump modules (Regex)", L"--regex=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_REGEX },
			{ L"Dump at base offset", L"--base=", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_HEXADDRESS },
			{ L"Ignore case", L"--ignore-case", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Carve memory sample", L"--memory", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Bypass sanity checks", L"--unsafe", CHECKTYPE_CHECKBOX, IDC_CHECK6, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Modify base to in-mem base address", L"--fix", CHECKTYPE_CHECKBOX, IDC_CHECK7, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		//enumfunc

		{ L"-- Process Memory --", L"", 0 },

		{ L"memmap", L"Print the memory map", 3,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX }
		}
		},

		{ L"memdump", L"Dump the addressable memory for a process\r\nDump directory will be created in the Volatility Workbench directory", 4,
		{
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_CHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_DIR },
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN2, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO4, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_REGEX }
		}
		},

		{ L"procdump", L"Dump a process to an executable file sample\r\nDump directory will be created in the Volatility Workbench directory", 7,
		{
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_CHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_DIR },
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN2, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO4, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_REGEX },
			{ L"Bypass sanity checks", L"--unsafe", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Carve memory sample", L"--memory", CHECKTYPE_CHECKBOX, IDC_CHECK6, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Modify base to in-mem base address", L"--fix", CHECKTYPE_CHECKBOX, IDC_CHECK7, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"vadinfo", L"Dump the VAD info", 4,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX },
			{ L"Containing address", L"--addr=", CHECKTYPE_RADIOBUTTON, IDC_RADIO4, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_HEXADDRESS }
		}
		},

		{ L"vadwalk", L"Walk the VAD tree", 4,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX },
			{ L"Containing address", L"--addr=", CHECKTYPE_RADIOBUTTON, IDC_RADIO4, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_HEXADDRESS }
		}
		},

		{ L"vadtree", L"Walk the VAD tree and display in tree format", 4,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX },
			{ L"Containing address", L"--addr=", CHECKTYPE_RADIOBUTTON, IDC_RADIO4, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_HEXADDRESS }
		}
		},

		{ L"vaddump", L"Dumps out the vad sections to a file\r\nDump directory will be created in the Volatility Workbench directory", 6,
		{
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_CHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_DIR },
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN2, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO4, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_REGEX },
			{ L"Dump with BASE AD", L"--base=", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT5, NULL, VALIDATION_HEXADDRESS },
			{ L"Maximum size", L"--max-size=", CHECKTYPE_CHECKBOX, IDC_CHECK6, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT6, NULL, VALIDATION_DIGIT }
		}
		},

		{ L"evtlogs", L"Extract Windows Event Logs (XP/2003 only)\r\nDump directory will be created in the Volatility Workbench directory", 3,
		{
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_CHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_DIR },
			{ L"Save raw .evt", L"--save-evt", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Verbose", L"--verbose", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"iehistory", L"Reconstruct Internet Explorer cache / history", 5,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX },
			{ L"Find LEAK records", L"--leak", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Find REDR records", L"--redr", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"-- Kernel Memory and Objects --", L"", 0 },

		{ L"modules", L"Print list of loaded modules", 1,
		{
			{ L"Physical Offset", L"--physical-offset", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"modscan", L"Pool scanner for kernel modules", 4,
		{
			{ L"Start scaning address", L"--start=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
			{ L"Length (in bytes)", L"--length=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DIGIT },
			{ L"Scan virtual space", L"--virtual", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Skip unalloc objects", L"--show-unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"moddump", L"Dump a kernel driver to an executable file sample\r\nDump directory will be created in the Volatility Workbench directory", 6,
		{
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_CHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_DIR },
			{ L"Dump modules (Regex)", L"--regex=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_REGEX },
			{ L"Ignore case", L"--ignore-case", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Carve memory sample", L"--memory", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Bypass sanity checks", L"--unsafe", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Modify base to in-mem", L"--fix", CHECKTYPE_CHECKBOX, IDC_CHECK6, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"ssdt", L"Display SSDT entries", 0 },

		{ L"driverscan", L"Pool scanner for driver objects", 4,
		{
			{ L"Start scaning address", L"--start=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
			{ L"Length (in bytes)", L"--length=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DIGIT },
			{ L"Scan virtual space", L"--virtual", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Skip unalloc objects", L"--show-unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"filescan", L"Pool scanner for file objects", 4,
		{
			{ L"Start scaning address", L"--start=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
			{ L"Length (in bytes)", L"--length=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DIGIT },
			{ L"Scan virtual space", L"--virtual", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Skip unalloc objects", L"--show-unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"mutantscan", L"Pool scanner for mutex objects", 5,
		{
			{ L"Start scaning address", L"--start=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
			{ L"Length (in bytes)", L"--length=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DIGIT },
			{ L"Scan virtual space", L"--virtual", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Skip unalloc objects", L"--show-unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Silent", L"--silent", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"symlinkscan", L"Pool scanner for symlink objects", 4,
		{
			{ L"Start scaning address", L"--start=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
			{ L"Length (in bytes)", L"--length=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DIGIT },
			{ L"Scan virtual space", L"--virtual", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Skip unalloc objects", L"--show-unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},
		
		{ L"thrdscan", L"Pool scanner for thread objects", 4,
		{
			{ L"Start scaning address", L"--start=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
			{ L"Length (in bytes)", L"--length=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DIGIT },
			{ L"Scan virtual space", L"--virtual", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Skip unalloc objects", L"--show-unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"dumpfiles", L"Extract memory mapped and cached files\r\nDump directory will be created in the Volatility Workbench directory", 7,
		{
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_CHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_DIR },
			{ L"Summary File", L"--summary-file=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DISABLED },
			{ L"Files match (Regex)", L"--regex=", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX },
			{ L"Process ID", L"--pid=", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN4, &gProcessList, VALIDATION_DISABLED },
			{ L"At addresses", L"--offset=", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT5, NULL, VALIDATION_HEXADDRESS },
			{ L"Include ext filename", L"--name", CHECKTYPE_CHECKBOX, IDC_CHECK6, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Relax safty constraints", L"--unsafe", CHECKTYPE_CHECKBOX, IDC_CHECK7, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
		}
		},

		{ L"unloadedmodules", L"Print list of unloaded modules", 0 },

		{ L"-- Networking --", L"", 0 },

		{ L"connections", L"This module follows the handle table in tcpip.sys and prints current connections", 1,
		{
			{ L"Physical Offset", L"--physical-offset", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"connscan", L"Pool scanner for tcp connections", 4,
		{
			{ L"Start scaning address", L"--start=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
			{ L"Length (in bytes)", L"--length=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DIGIT },
			{ L"Scan virtual space", L"--virtual", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Skip unalloc objects", L"--show-unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"sockets", L"Print list of open sockets", 1,
		{
			{ L"Physical Offset", L"--physical-offset", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"sockscan", L"Pool scanner for tcp socket objects", 4,
		{
			{ L"Start scaning address", L"--start=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
			{ L"Length (in bytes)", L"--length=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DIGIT },
			{ L"Scan virtual space", L"--virtual", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Skip unalloc objects", L"--show-unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"netscan", L"Scan a Vista (or later) image for connections and sockets", 4,
		{
			{ L"Start scaning address", L"--start=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
			{ L"Length (in bytes)", L"--length=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DIGIT },
			{ L"Scan virtual space", L"--virtual", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Skip unalloc objects", L"--show-unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"-- Registry --", L"", 0 },

		{ L"hivescan", L"Pool scanner for registry hives", 4,
		{
			{ L"Start scaning address", L"--start=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
			{ L"Length (in bytes)", L"--length=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DIGIT },
			{ L"Scan virtual space", L"--virtual", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Skip unalloc objects", L"--show-unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"hivelist", L"Print list of registry hives", 4,
		{
			{ L"Start scaning address", L"--start=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
			{ L"Length (in bytes)", L"--length=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DIGIT },
			{ L"Scan virtual space", L"--virtual", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Skip unalloc objects", L"--show-unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"printkey", L"Print a registry key, and its subkeys and values", 6,
		{
			{ L"Registry Key", L"--key=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_DISABLED },
			{ L"Hive Offset (virtual)", L"--hive-offset=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Start scaning address", L"--start=", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_HEXADDRESS },
			{ L"Length (in bytes)", L"--length=", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_DIGIT },
			{ L"Scan virtual space", L"--virtual", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Skip unalloc objects", L"--show-unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK6, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"hivedump", L"Prints out a hive", 1,
		{
			{ L"Hive Offset (virtual)", L"--hive-offset=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_CHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS }
		}
		},

		{ L"hashdump", L"Dumps passwords hashes (LM/NTLM) from memory", 2,
		{
			{ L"SYS Offset (virtual)", L"--sys-offset=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
			{ L"SAM Offset (virtual)", L"--sam-offset=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS }
		}
		},

		{ L"lsadump", L"Dump (decrypted) LSA secrets from the registry", 2,
		{
			{ L"SYS Offset (virtual)", L"--sys-offset=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
			{ L"SEC Offset (virtual)", L"--sec-offset=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS }
		}
		},

		{ L"userassist", L"Print userassist registry keys and information", 1,
		{
			{ L"Hive Offset (virtual)", L"--hive-offset=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS }
		}
		},

		{ L"shellbags", L"Prints ShellBags info", 1,
		{
			{ L"Machine Name", L"--machine=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"shimcache", L"Parses the Application Compatibility Shim Cache registry key", 0 },

		{ L"getservicesids", L"Get the names of services in the Registry and return Calculated SID", 0 },

		{ L"dumpregistry", L"Dumps registry files out to disk\r\nDump directory will be created in the Volatility Workbench directory", 2,
		{
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_CHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_DIR },
			{ L"Hive Offset (virtual)", L"--hive-offset=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS }
		}
		},

		{ L"-- Crash Dumps, Hibernation, and Conversion --", L"", 0 },

		{ L"crashinfo", L"Dump crash-dump information", 0 },

		{ L"hibinfo", L"Dump hibernation file information", 0 },

		{ L"imagecopy", L"Copies a physical address space out as a raw DD image", 2,
		{
			{ L"Output Image", L"--output-image=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_CHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_DISABLED },
			{ L"Block Size (in bytes)", L"--blocksize=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DIGIT }
		}
		},

		{ L"raw2dmp", L"Converts a physical memory sample to a windbg crash dump", 2,
		{
			{ L"Output Image", L"--output-image=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_CHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_DISABLED },
			{ L"Block Size (in bytes)", L"--blocksize=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DIGIT }
		}
		},

		{ L"vboxinfo", L"Dump virtualbox information", 0 },

		{ L"vmwareinfo", L"Dump VMware VMSS/VMSN information\r\nDump directory will be created in the Volatility Workbench directory", 2,
		{
			{ L"Screenshot Dir Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_DIR },
			{ L"Verbose", L"--verbose", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"hpakinfo", L"Info on an HPAK file", 0 },

		{ L"hpakextract", L"Extract physical memory from an HPAK file", 0 },

		{ L"-- File System --", L"", 0 },

		{ L"mbrparser", L"Scans for and parses potential Master Boot Records (MBRs)", 9,
		{
			{ L"Offset of MBR", L"--offset=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
			{ L"Hash of bootcode", L"--hash=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DISABLED },
			{ L"Hash of full bootcode", L"--fullhash=", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_DISABLED },
			{ L"Offset disassembly", L"--disoffset=", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_HEXADDRESS },
			{ L"Maximum Levenshtein distance", L"--maxdistance=", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT5, NULL, VALIDATION_DISABLED },
			{ L"Disk / extracted MBR", L"--disk=", CHECKTYPE_CHECKBOX, IDC_CHECK6, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT6, NULL, VALIDATION_DISABLED },
			{ L"Output HEX of Bootcode", L"--hex", CHECKTYPE_CHECKBOX, IDC_CHECK7, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Don't check partitions", L"--nocheck", CHECKTYPE_CHECKBOX, IDC_CHECK9, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Start header at zero", L"--zerostart", CHECKTYPE_CHECKBOX, IDC_CHECK10, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},

		{ L"mftparser", L"Scans for and parses potential MFT entries", 6,
		{
			{ L"Machine Name", L"--machine=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_DISABLED },
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DISABLED },
			{ L"MFT Entry Size", L"--entrysize=", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_DIGIT },
			{ L"MFT Offset (physical)", L"--offset=", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_HEXADDRESS },
			{ L"Don't check partitions", L"--nocheck", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Debugging messages", L"--debugout", CHECKTYPE_CHECKBOX, IDC_CHECK6, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED }
		}
		},


        // Mac Commands
		{ L"-- Processes --", L"", 0 },

		{ L"mac_pslist", L"List running processes", 2,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"Task Name", L"--task=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_REGEX },
		}
		},

		{ L"mac_tasks", L"List active tasks", 2,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"Task Name", L"--task=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_REGEX },
		}
	    },

		{ L"mac_pstree", L"Show parent/child relationship of processes", 2,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"Task Name", L"--task=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_REGEX },
	    }
	    },

		{ L"mac_lsof", L"Lists per-process opened files", 2,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"Task Name", L"--task=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_REGEX },
		}
		},

		{ L"mac_pgrp_hash_table", L"Walks the process group hash table", 2,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"Task Name", L"--task=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_REGEX },
		}
		},

		{ L"mac_pid_hash_table", L"Walks the pid hash table", 2,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"Task Name", L"--task=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_REGEX },
		}
		},

		{ L"mac_psaux", L"Prints processes with arguments in user land", 2,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"Task Name", L"--task=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_REGEX },
		}
		},

		{ L"mac_dead_procs", L"Prints terminated/de-allocated processes", 2,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"Task Name", L"--task=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_REGEX },
		}
		},

		{ L"mac_psxview", L"Find hidden processes with various process listings", 0 },


		{ L"-- Process Memory --", L"", 0 },

		{ L"mac_proc_maps", L"Gets memory maps of processes", 2,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"Task Name", L"--task=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_REGEX },
		}
		},

		{ L"mac_dump_maps", L"Dumps memory ranges of process(es), optionally including pages in compressed swap",  3,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"Task Name", L"--task=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_REGEX },
			{ L"MAP Address", L"--map-address=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_HEXADDRESS },
		}
		},


		{ L"-- Kernel Memory and Objects --", L"", 0 },

		{ L"mac_list_sessions", L"Enumerates sessions", 2,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"Task Name", L"--task=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_REGEX },
		}
		},

		{ L"mac_list_zones", L"Prints active zones", 0 },

		{ L"mac_lsmod", L"Lists loaded kernel modules",  1,
		{
			{ L"Address", L"--addr=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
		}
		},

		{ L"mac_mount", L"Prints mounted device information", 0 },


		{ L"-- Networking --", L"", 0 },

		{ L"mac_arp", L"Prints the arp table", 0 },

		{ L"mac_ifconfig", L"Lists network interface information for all devices", 0 },

		{ L"mac_netstat", L"Lists active per-process network connections", 0 },

		{ L"mac_route", L"Prints the routing table", 0 },


		{ L"-- Malware/Rootkits --", L"", 0 },

		{ L"mac_check_sysctl", L"Checks for unknown sysctl handlers", 0 },

		{ L"mac_check_syscalls", L"Checks to see if system call table entries are hooked",  1,
		{
			{ L"Syscall Indexes", L"--syscall-indexes=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_REGEX },
		}
		},

		{ L"mac_check_trap_table", L"Checks to see if mach trap table entries are hooked", 0 },

		{ L"mac_ip_filters", L"Reports any hooked IP filters",  1,
		{
			{ L"Address", L"--addr=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
		}
		},

		{ L"mac_notifiers", L"Detects rootkits that add hooks into I/O Kit", 1,
		{
			{ L"Address", L"--addr=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
		}
		},

		{ L"mac_trustedbsd", L"Lists malicious trustedbsd policies", 1,
		{
			{ L"Address", L"--addr=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_HEXADDRESS },
		}
		},

		{ L"-- System Information --", L"", 0 },

		{ L"mac_dmesg", L"Prints the kernel debug buffer", 0 },

		{ L"mac_find_aslr_shift", L"Find the ASLR shift value for 10.8+ images", 0 },

		{ L"mac_machine_info", L"Prints machine information about the sample", 0 },

		{ L"mac_version", L"Prints the Mac version", 0 },

		{ L"mac_print_boot_cmdline", L"Prints kernel boot arguments", 0 },


		{ L"-- Miscellaneous --", L"", 0 },

		{ L"mac_yarascan", L"Scan memory for yara signatures", 10,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX },
			{ L"Yara rules (String)", L"--yara-rules=", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_CHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_DISABLED },
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT5, NULL, VALIDATION_DIR },
			{ L"Maximum size", L"--max-size=", CHECKTYPE_CHECKBOX, IDC_CHECK6, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT6, NULL, VALIDATION_DIGIT },
			{ L"Scan process and kernel mem", L"--all", CHECKTYPE_CHECKBOX, IDC_CHECK7, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Case insensitive", L"--case", CHECKTYPE_CHECKBOX, IDC_CHECK8, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Scan kernel modules", L"--kernel", CHECKTYPE_CHECKBOX, IDC_CHECK9, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Match wide", L"--wide", CHECKTYPE_CHECKBOX, IDC_CHECK10, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
		}
		},
	

        // Linux Commands
		{ L"-- Processes --", L"", 0 },

		{ L"linux_pslist", L"Gather active tasks by walking the task_struct->task list", 1,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
		}
		},

		{ L"linux_psaux", L"Gathers processes along with full command line and start time", 1,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
		}
		},

		{ L"linux_pstree", L"Shows the parent/child relationship between processes", 1,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
		}
		},

		{ L"linux_pslist_cache", L"Gather tasks from the kmem_cache", 2,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"Show Unallocated", L"--unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
		}
		},

		{ L"linux_pidhashtable", L"Enumerates processes through the PID hash table", 1,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
		}
		},

		{ L"linux_psxview", L"Find hidden processes with various process listings", 0 },

		{ L"linux_lsof", L"Lists file descriptors and their path", 1,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
		}
		},


		{ L"-- Process Memory --", L"", 0 },

		{ L"linux_memmap", L"Dumps the memory map for linux tasks", 1,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
		}
		},

		{ L"linux_proc_maps", L"Gathers process memory maps",  1,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
		}
		},

		{ L"linux_dump_map", L"Writes selected memory mappings to disk", 3,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"VMA Starting Address", L"--vma=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_DIR },
		}
		},

		{ L"linux_bash", L"Recover bash history from bash process memory",  3,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"Print Unallocated Entries", L"--printunalloc", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Scan All Processes", L"--scan_all", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
		}
		},

		{ L"-- Kernel Memory and Objects --", L"", 0 },

		{ L"linux_lsmod", L"Gather loaded kernel modules", 4,
		{
			{ L"Show Section Addresses", L"--sections", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Show Module Parameters", L"--params", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"BASE Address", L"--base=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_HEXADDRESS },
			{ L"IDC Path", L"--idc=", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_DISABLED },
		}
		},

		{ L"linux_moddump", L"Extract loaded kernel modules", 4, 
		{
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_DIR },
			{ L"Regex to Match", L"--regex=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_REGEX },
			{ L"BASE Address", L"--base=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_HEXADDRESS },
			{ L"Ignore Case", L"--ignore-case", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
		}
		},

		{ L"linux_tmpfs", L"Recovers tmpfs filesystems from memory",  3,
		{
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_DIR },
			{ L"List Superblocks", L"--list_sbs", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Process Superblock", L"--sb=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX },
		}
		},


		{ L"-- Rootkit Detection --", L"", 0 },

		{ L"linux_check_afinfo", L"Verifies the operation function pointers of network protocols", 0 },

		{ L"linux_check_tty", L"Checks tty devices for hooks", 0 },

		{ L"linux_check_creds", L"Checks if any processes are sharing credential structures", 1,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
		}
		},

		{ L"linux_check_fop", L"Check file operation structures for rootkit modifications", 1,
		{
			{ L"Inode to check", L"--inode=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_REGEX },
		}
		},

		{ L"linux_check_idt", L"Checks if the IDT has been altered", 0 },

		{ L"linux_check_syscall", L"Checks if the system call table has been altered", 0 },

		{ L"linux_check_modules", L"Compares module list to sysfs info, if available", 0 },


		{ L"-- Networking --", L"", 0 },

		{ L"linux_arp", L"Prints the arp table", 0 },

		{ L"linux_ifconfig", L"Gathers active interfaces", 0 },

		{ L"linux_route_cache", L"Recovers the routing cache from memory", 1, 
		{
			{ L"Resolve DNS Names", L"--resolve", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
		}
		},

		{ L"linux_netstat", L"Lists open sockets", 2, 
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"Ignore UNIX Sockets", L"--ignore_unix", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
		}
		},

		{ L"linux_pkt_queues", L"Writes per-process packet queues out to disk", 3,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"Ignore UNIX Sockets", L"--ignore_unix", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_DIR },
		}
		},

		{ L"linux_sk_buff_cache", L"Recovers packets from the sk_buff kmem_cache", 2,
		{
			{ L"Show Unallocated", L"--unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_DIR },
		}
		},


		{ L"-- System Information --", L"", 0 },

		{ L"linux_cpuinfo", L"Prints info about each active processor", 0 },

		{ L"linux_dmesg", L"Gather dmesg buffer", 0 },

		{ L"linux_iomem", L"Provides output similar to /proc/iomem", 0 },

		{ L"linux_slabinfo", L"Mimics /proc/slabinfo on a running machine", 0 },

		{ L"linux_mount", L"Gather mounted fs/devices", 0 },

		{ L"linux_mount_cache", L"Gather mounted fs/devices from kmem_cache", 1,
		{
			{ L"Show Unallocated", L"--unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
		}
		},

		{ L"linux_dentry_cache", L"Gather files from the dentry cache", 1,
		{
			{ L"Show Unallocated", L"--unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
		}
		},

		{ L"linux_find_file", L"Lists and recovers files from memory", 3,
		{
			{ L"Find in Path", L"--find=", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT1, NULL, VALIDATION_DISABLED },
			{ L"Inode", L"--inode=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_REGEX },
			{ L"List Files", L"--listfiles", CHECKTYPE_CHECKBOX, IDC_CHECK3, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
		}
		},

		{ L"linux_vma_cache", L"Gather VMAs from the vm_area_struct cache", 1,
		{
			{ L"Show Unallocated", L"--unallocated", CHECKTYPE_CHECKBOX, IDC_CHECK1, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
		}
		},

		{ L"-- Miscellaneous --", L"", 0 },

		{ L"linux_yarascan", L"Scan memory for yara signatures", 10,
		{
			{ L"Process ID", L"--pid=", CHECKTYPE_RADIOBUTTON, IDC_RADIO1, BST_UNCHECKED, VALUETYPE_COMBOBOX, IDC_DROPDOWN1, &gProcessList, VALIDATION_DISABLED },
			{ L"EPROCESS Offset", L"--offset=", CHECKTYPE_RADIOBUTTON, IDC_RADIO2, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT2, NULL, VALIDATION_HEXADDRESS },
			{ L"Process Name (Regex)", L"--name=", CHECKTYPE_RADIOBUTTON, IDC_RADIO3, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT3, NULL, VALIDATION_REGEX },
			{ L"Yara rules (String)", L"--yara-rules=", CHECKTYPE_CHECKBOX, IDC_CHECK4, BST_CHECKED, VALUETYPE_EDITBOX, IDC_EDIT4, NULL, VALIDATION_DISABLED },
			{ L"Dump Folder Name", L"--dump-dir=", CHECKTYPE_CHECKBOX, IDC_CHECK5, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT5, NULL, VALIDATION_DIR },
			{ L"Maximum size", L"--max-size=", CHECKTYPE_CHECKBOX, IDC_CHECK6, BST_UNCHECKED, VALUETYPE_EDITBOX, IDC_EDIT6, NULL, VALIDATION_DIGIT },
			{ L"Scan process and kernel mem", L"--all", CHECKTYPE_CHECKBOX, IDC_CHECK7, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Case insensitive", L"--case", CHECKTYPE_CHECKBOX, IDC_CHECK8, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Scan kernel modules", L"--kernel", CHECKTYPE_CHECKBOX, IDC_CHECK9, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
			{ L"Match wide", L"--wide", CHECKTYPE_CHECKBOX, IDC_CHECK10, BST_UNCHECKED, VALUETYPE_NOVALUE, 0, NULL, VALIDATION_DISABLED },
		}
		}
	};


	VOLATILITY_PROFILE_TYPEDEF		gProfiles[NUM_PROFILES] =
	{
		VOLATILITY_PROFILE(L"",	L"-- Select Profile --"),
		//VOLATILITY_PROFILE(L"Win10x64_18362",	L"Windows 10 64bit build 18362"), not in volatility yet
		VOLATILITY_PROFILE(L"Win10x64_17763",	L"Windows 10 64bit build 17763"),
		VOLATILITY_PROFILE(L"Win10x64_17134",	L"Windows 10 64bit build 17134"),
		VOLATILITY_PROFILE(L"Win10x64_10240_17770",	L"Windows 10 64bit build 10240_17770"),
		VOLATILITY_PROFILE(L"Win10x64_16299",   L"Windows 10 64bit build 16299"),
		VOLATILITY_PROFILE(L"Win10x64_15063",	L"Windows 10 64bit build 15063"),
		VOLATILITY_PROFILE(L"Win10x64_14393",	L"Windows 10 64bit build 14393"),
		VOLATILITY_PROFILE(L"Win10x64_10586",	L"Windows 10 64bit build 10586"),
		VOLATILITY_PROFILE(L"Win10x64",			L"Windows 10 64bit base version"),
		//VOLATILITY_PROFILE(L"Win10x96_18362",	L"Windows 10 32bit build 18362"),  not in volatility yet
		VOLATILITY_PROFILE(L"Win10x86_17763",	L"Windows 10 32bit build 17763"),
		VOLATILITY_PROFILE(L"Win10x86_17134",	L"Windows 10 32bit build 17134"),
		VOLATILITY_PROFILE(L"Win10x86_10240_17770",	L"Windows 10 32bit build 10240_17770"),
		VOLATILITY_PROFILE(L"Win10x86_16299",	L"Windows 10 32bit build 16299"),
		VOLATILITY_PROFILE(L"Win10x86_15063",	L"Windows 10 32bit build 15063"),
		VOLATILITY_PROFILE(L"Win10x86_14393",	L"Windows 10 32bit build 14393"),
		VOLATILITY_PROFILE(L"Win10x86_10586",	L"Windows 10 32bit build 10586"),
		VOLATILITY_PROFILE(L"Win10x86",			L"Windows 10 32bit base version"),
		VOLATILITY_PROFILE(L"Win81U1x64",		L"Windows 8.1 64bit base version"),
		VOLATILITY_PROFILE(L"Win81U1x86",		L"Windows 8.1 32bit base version"),
		VOLATILITY_PROFILE(L"Win8SP1x64_18340", L"Windows 8 64bit build 18340"),
		VOLATILITY_PROFILE(L"Win8SP1x64",		L"Windows 8 64bit service pack 1"),
		VOLATILITY_PROFILE(L"Win8SP0x64",		L"Windows 8 64bit base version"),
		VOLATILITY_PROFILE(L"Win8SP1x86",		L"Windows 8 32bit service pack 1"),
		VOLATILITY_PROFILE(L"Win8SP0x86",		L"Windows 8 32bit base version"),
		VOLATILITY_PROFILE(L"Win7SP1x64_23418", L"Windows 7 64bit build 23418"),
		VOLATILITY_PROFILE(L"Win7SP1x64",		L"Windows 7 64bit service pack 1"),
		VOLATILITY_PROFILE(L"Win7SP0x64",		L"Windows 7 64bit base version"),
		VOLATILITY_PROFILE(L"Win7SP1x86_23418", L"Windows 7 32bit build 23418"),
		VOLATILITY_PROFILE(L"Win7SP1x86",		L"Windows 7 32bit service pack 1"),
		VOLATILITY_PROFILE(L"Win7SP0x86",		L"Windows 7 32bit base version"),
		VOLATILITY_PROFILE(L"VistaSP2x64",		L"Windows Vista 64bit service pack 2"),
		VOLATILITY_PROFILE(L"VistaSP1x64",		L"Windows Vista 64bit service pack 1"),
		VOLATILITY_PROFILE(L"VistaSP0x86",		L"Windows Vista 32bit base version"),
		VOLATILITY_PROFILE(L"VistaSP2x86",		L"Windows Vista 32bit service pack 2"),
		VOLATILITY_PROFILE(L"VistaSP1x86",		L"Windows Vista 32bit service pack 1"),
		VOLATILITY_PROFILE(L"WinXPSP2x64",		L"Windows XP 64bit service pack 2"),
		VOLATILITY_PROFILE(L"WinXPSP1x64",		L"Windows XP 64bit service pack 1"),
		VOLATILITY_PROFILE(L"WinXPSP3x86",		L"Windows XP 32bit service pack 3"),
		VOLATILITY_PROFILE(L"WinXPSP2x86",		L"Windows XP 32bit service pack 2"),
		VOLATILITY_PROFILE(L"Win2016x64_14393", L"Windows 2016 64bit build 14393"),
		VOLATILITY_PROFILE(L"Win2012x64",		L"Windows 2012 64bit base version"),
		VOLATILITY_PROFILE(L"Win2012R2x64_18340", L"Windows 2012R2 64bit build 18430"),
		VOLATILITY_PROFILE(L"Win2012R2x64",		L"Windows 2012R2 64bit base version"),
		VOLATILITY_PROFILE(L"Win2008SP2x64",	L"Windows 2008 64bit service pack 2"),
		VOLATILITY_PROFILE(L"Win2008SP1x64",	L"Windows 2008 64bit service pack 1"),
		VOLATILITY_PROFILE(L"Win2008SP2x86",	L"Windows 2008 32bit base version"),
		VOLATILITY_PROFILE(L"Win2008SP1x86",	L"Windows 2008 32bit service pack 1"),
		VOLATILITY_PROFILE(L"Win2008R2SP1x64_23418", L"Windows 2008 64bit build 23418"),
		VOLATILITY_PROFILE(L"Win2008R2SP1x64",	L"Windows 2008R2 64bit service pack 1"),
		VOLATILITY_PROFILE(L"Win2008R2SP0x64",	L"Windows 2008R2 64bit base version"),
		VOLATILITY_PROFILE(L"Win2003SP2x64",	L"Windows 2003 64bit service pack 2"),
		VOLATILITY_PROFILE(L"Win2003SP1x64",	L"Windows 2003 64bit service pack 1"),
		VOLATILITY_PROFILE(L"Win2003SP0x64",	L"Windows 2003 64bit base version"),
		VOLATILITY_PROFILE(L"Win2003SP2x86",	L"Windows 2003 32bit service pack 2"),
		VOLATILITY_PROFILE(L"Win2003SP1x86",	L"Windows 2003 32bit service pack 1"),
		VOLATILITY_PROFILE(L"Win2003SP0x86",	L"Windows 2003 32bit base version"),
	};

	PROCESS_INFORMATION gProcessInfo;
	bool		gCommandRunning = false;
	wchar_t		gProfile[MAX_PROFILE_LEN];
	wchar_t		gKDBG_Addr[MAX_KDBG_ADDR_LEN];


#else

	extern		HWND				ghMainWnd;
	extern		char				lastURL[MAX_URL_LEN];;
	extern		wchar_t				fullpath[MAXFILENAME + MAXDIRNAME];
	extern		wchar_t				szDefaultLogName[MAXFILENAME];
	extern		wchar_t				RunDirName[MAXDIRNAME];
	extern		wchar_t				gFileName[MAXFILENAME + MAXDIRNAME];
	extern		unsigned int		gControlsList[17];
	extern		VOLATILITY_CMD_TYPEDEF			gCommands[NUM_WIN_COMMANDS];
	extern		VOLATILITY_PROFILE_TYPEDEF		gProfiles[NUM_PROFILES];
	extern		PROCESS_INFORMATION gProcessInfo;
	extern		bool				gCommandRunning;
	extern		wchar_t				gProfile;
	extern		wchar_t				gKDBG_Addr;
#endif

// Prototypes
	HANDLE SpawnWithRedirect(wchar_t *ProgName, PROCESS_INFORMATION *ProcessInfo);

#endif //VOLATILITY_WORKBENCH_H